@@ -53,12 +53,12 @@ module Asciidoctor
   module SafeMode
     # A safe mode level that disables any of the security features enforced
     # by Asciidoctor (Ruby is still subject to its own restrictions).
-    UNSAFE = 0;
+    UNSAFE = 0
 
     # A safe mode level that closely parallels safe mode in AsciiDoc. This value
     # prevents access to files which reside outside of the parent directory of
     # the source file and disables any macro other than the include::[] directive.
-    SAFE = 1;
+    SAFE = 1
 
     # A safe mode level that disallows the document from setting attributes
     # that would affect the conversion of the document, in addition to all the
@@ -66,19 +66,19 @@ module Asciidoctor
     # changing the backend or source-highlighter using an attribute defined
     # in the source document header. This is the most fundamental level of
     # security for server deployments (hence the name).
-    SERVER = 10;
+    SERVER = 10
 
     # A safe mode level that disallows the document from attempting to read
     # files from the file system and including the contents of them into the
     # document, in additional to all the security features of SafeMode::SERVER.
     # For instance, this level disallows use of the include::[] directive and the
     # embedding of binary content (data uri), stylesheets and JavaScripts
-    # referenced by the document.(Asciidoctor and trusted extensions may still
+    # referenced by the document. (Asciidoctor and trusted extensions may still
     # be allowed to embed trusted content into the document).
     #
     # Since Asciidoctor is aiming for wide adoption, this level is the default
     # and is recommended for server deployments.
-    SECURE = 20;
+    SECURE = 20
 
     # A planned safe mode level that disallows the use of passthrough macros and
     # prevents the document from setting any known attributes, in addition to all
@@ -86,9 +86,9 @@ module Asciidoctor
     #
     # Please note that this level is not currently implemented (and therefore not
     # enforced)!
-    #PARANOID = 100;
+    #PARANOID = 100
 
-    @names_by_value = {}.tap {|accum| (constants false).each {|sym| accum[const_get sym, false] = sym.to_s.downcase } }
+    @names_by_value = (constants false).map {|sym| [(const_get sym), sym.to_s.downcase] }.sort {|(a), (b)| a <=> b }.to_h
 
     def self.value_for_name name
       const_get name.upcase, false
@@ -136,8 +136,8 @@ module Asciidoctor
     # Compliance value: true
     define :underline_style_section_titles, true
 
-    # Asciidoctor will unwrap the content in a preamble
-    # if the document has a title and no sections.
+    # Asciidoctor will unwrap the content in a preamble if the document has a
+    # title and no sections, then discard the empty preamble.
     # Compliance value: false
     define :unwrap_standalone_preamble, true
 
@@ -148,7 +148,7 @@ module Asciidoctor
     # Compliance value: 'drop-line'
     define :attribute_missing, 'skip'
 
-    # AsciiDoc drops lines that contain an attribute unassignemnt.
+    # AsciiDoc drops lines that contain an attribute unassignment.
     # This behavior may need to be tuned depending on the circumstances.
     # Compliance value: 'drop-line'
     define :attribute_undefined, 'drop-line'
@@ -215,7 +215,7 @@ module Asciidoctor
   URI_READ_MODE = FILE_READ_MODE
 
   # The mode to use when opening a file for writing
-  FILE_WRITE_MODE = RUBY_ENGINE_OPAL ? 'w' : 'w:utf-8'
+  FILE_WRITE_MODE = RUBY_ENGINE_OPAL ? 'w' : 'wb:utf-8'
 
   # The default document type
   # Can influence markup generated by the converters
@@ -296,7 +296,7 @@ module Asciidoctor
   DELIMITED_BLOCK_TAILS = {}.tap {|accum| DELIMITED_BLOCKS.each_key {|k| accum[k] = k[k.length - 1] if k.length == 4 } }
 
   # NOTE the 'figure' key as a string is historical and used by image blocks
-  CAPTION_ATTR_NAMES = { example: 'example-caption', 'figure' => 'figure-caption', listing: 'listing-caption', table: 'table-caption' }
+  CAPTION_ATTRIBUTE_NAMES = { example: 'example-caption', 'figure' => 'figure-caption', listing: 'listing-caption', table: 'table-caption' }
 
   LAYOUT_BREAK_CHARS = {
     '\'' => :thematic_break,
@@ -332,7 +332,7 @@ module Asciidoctor
 
   LIST_CONTINUATION = '+'
 
-  # NOTE AsciiDoc Python allows + to be preceded by TAB; Asciidoctor does not
+  # NOTE AsciiDoc.py allows + to be preceded by TAB; Asciidoctor does not
   HARD_LINE_BREAK = ' +'
 
   LINE_CONTINUATION = ' \\'
@@ -357,9 +357,35 @@ module Asciidoctor
 
   FONT_AWESOME_VERSION = '4.7.0'
 
-  HIGHLIGHT_JS_VERSION = '9.15.6'
+  HIGHLIGHT_JS_VERSION = '9.18.3'
 
-  MATHJAX_VERSION = '2.7.5'
+  MATHJAX_VERSION = '2.7.9'
+
+  DEFAULT_ATTRIBUTES = {
+    'appendix-caption' => 'Appendix',
+    'appendix-refsig' => 'Appendix',
+    'caution-caption' => 'Caution',
+    'chapter-refsig' => 'Chapter',
+    #'encoding' => 'UTF-8',
+    'example-caption' => 'Example',
+    'figure-caption' => 'Figure',
+    'important-caption' => 'Important',
+    'last-update-label' => 'Last updated',
+    #'listing-caption' => 'Listing',
+    'note-caption' => 'Note',
+    'part-refsig' => 'Part',
+    #'preface-title' => 'Preface',
+    'prewrap' => '',
+    'sectids' => '',
+    'section-refsig' => 'Section',
+    'table-caption' => 'Table',
+    'tip-caption' => 'Tip',
+    'toc-placement' => 'auto',
+    'toc-title' => 'Table of Contents',
+    'untitled-label' => 'Untitled',
+    'version-label' => 'Version',
+    'warning-caption' => 'Warning',
+  }
 
   # attributes which be changed throughout the flow of the document (e.g., sectnums)
   FLEXIBLE_ATTRIBUTES = ['sectnums']
@@ -433,9 +459,9 @@ module Asciidoctor
       [:emphasis, :unconstrained, /\\?(?:\[([^\]]+)\])?__(#{CC_ALL}+?)__/m],
       # _emphasis_
       [:emphasis, :constrained, /(^|[^#{CC_WORD};:}])(?:\[([^\]]+)\])?_(\S|\S#{CC_ALL}*?\S)_(?!#{CG_WORD})/m],
-      # ##mark## (referred to in AsciiDoc Python as unquoted)
+      # ##mark## (referred to in AsciiDoc.py as unquoted)
       [:mark, :unconstrained, /\\?(?:\[([^\]]+)\])?##(#{CC_ALL}+?)##/m],
-      # #mark# (referred to in AsciiDoc Python as unquoted)
+      # #mark# (referred to in AsciiDoc.py as unquoted)
       [:mark, :constrained, /(^|[^#{CC_WORD}&;:}])(?:\[([^\]]+)\])?#(\S|\S#{CC_ALL}*?\S)#(?!#{CG_WORD})/m],
       # ^superscript^
       [:superscript, :unconstrained, /\\?(?:\[([^\]]+)\])?\^(\S+?)\^/],
@@ -469,8 +495,8 @@ module Asciidoctor
     # (TM)
     [/\\?\(TM\)/, '&#8482;', :none],
     # foo -- bar (where either space character can be a newline)
-    # NOTE this necessarily drops the newline if it appears at end of line
-    [/(^|\n| |\\)--( |\n|$)/, '&#8201;&#8212;&#8201;', :none],
+    # NOTE this necessarily drops the newline if replacement appears at end of line
+    [/(?: |\n|^|\\)--(?: |\n|$)/, '&#8201;&#8212;&#8201;', :none],
     # foo--bar
     [/(#{CG_WORD})\\?--(?=#{CG_WORD})/, '&#8212;&#8203;', :leading],
     # ellipsis
@@ -511,8 +537,8 @@ module Asciidoctor
   end unless RUBY_ENGINE == 'opal'
 
   unless RUBY_ENGINE == 'opal'
-    autoload :SyntaxHighlighter, %(#{LIB_DIR}/asciidoctor/syntax_highlighter)
-    autoload :Timings, %(#{LIB_DIR}/asciidoctor/timings)
+    autoload :SyntaxHighlighter, %(#{__dir__}/asciidoctor/syntax_highlighter)
+    autoload :Timings, %(#{__dir__}/asciidoctor/timings)
   end
 end
 
